
/* Copyright  2014 Pierre Vidal <p.terkal@gmail.com>
 * This program is free software. It comes without no warranty.
 * You can redistribute it and/or modify it under the terms of the
 * Do What The Fuck You Want To Public License, Version 2, as published by Sam Hocevar.
 * See the COPYING file or http://www.wtfpl.net/ for more details. */

#include "main.h"

#include <stdio.h> // sprintf()

#define GRID_X_OFFSET 200
#define GRID_Y_OFFSET 140
#define GRID_SIZE 8
#define GRID_MULTIPL 4
#define PART_SIZE 9
#define GRID_CASE_SIZE PART_SIZE * GRID_MULTIPL

enum part_type {
    cockpit_1x1,
    cockpit_1x2,
    cockpit_2x1,
    cockpit_2x2,
    thruster_1x1,
    thruster_1x2,
    thruster_2x2,
    gun_1x1,
    leftgun_1x1,
    rightgun_1x1,
    larboardgun_1x1, // left
    starboardgun_1x1, // right
    reargun_1x1,
    thrustgun_1x2,
    triplegun_2x1,
    mushinegun_1x3,
    // mushotgun, // TODO
    sniper_1x2,
    aiming_lazer_1x1,
    lazer_1x3,
    rostrum_1x2,
    leftspike_1x2,
    rightspike_1x2,
    part_type_max
};

struct part {
    enum part_type type;
    int x;
    int y;
// X and Y are in number of square, not in pixel.
};

// TODO : How to store and process thruster and weapon blocking?
struct part_model {
    char* name;
    char* path;
    SDL_Surface* surface;
    SDL_Texture* texture;
    int w;
    int h;
    int hp;
    float weight;
    float power;
    float speed;
};

struct archip {
    int nb_parts;
    struct part part[MAX_PARTS];
};

char* get_genebind_name(enum gene_keybind i) {
    return(local.console_keybind[i]);
}

char* get_genebind_hint(enum gene_keybind i) {
    return(local.console_hint[i]);
}

void build_default_keybind(SDL_Keycode keybind[gene_keybind_max]) {
    for(enum gene_keybind i=0; i<gene_keybind_max; i++) {
        switch(i) {
            case gene_moveup : keybind[i] = SDLK_UP; break;
            case gene_movedown : keybind[i] = SDLK_DOWN; break;
            case gene_moveleft : keybind[i] = SDLK_LEFT; break;
            case gene_moveright : keybind[i] = SDLK_RIGHT; break;
            case gene_scrollup : keybind[i] = SDLK_KP_MINUS; break;
            case gene_scrolldown : keybind[i] = SDLK_KP_PLUS; break;
            case gene_raiseup : keybind[i] = SDLK_KP_DIVIDE; break;
            case gene_pushdown : keybind[i] = SDLK_KP_MULTIPLY; break;
            case gene_add : keybind[i] = SDLK_EQUALS; break;
            case gene_del : keybind[i] = SDLK_EXCLAIM; break;
            case gene_changenext : keybind[i] = SDLK_TAB; break;
            case gene_changeprev : keybind[i] = SDLK_BACKSPACE; break;
            case gene_validate : keybind[i] = SDLK_RETURN; break;
            case gene_keybind_max : break;
        }
    }
}

void gene_keybind_menu(SDL_Renderer* renderer, SDL_Keycode keybind[gene_keybind_max]) {
    SDL_Event event;
    SDL_Texture* texture;
    SDL_Rect dst;
    int stop = false;
    int waitkey = false;
    enum gene_keybind selector = 0;
    char buf[BUFSIZ];

    while(!stop) {

        // Process input.

        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                if(event.key.keysym.sym == SDLK_ESCAPE) {
                    if(waitkey) {
                        waitkey = false;
                    } else {
                        stop = true;
                    }
                } else if(waitkey && event.key.keysym.sym != SDLK_F1 && event.key.keysym.sym != SDLK_F2) {
                    keybind[selector] = event.key.keysym.sym;
                    waitkey = false;
                } else {
                    switch(event.key.keysym.sym) {
                        case SDLK_UP:
                            if(selector > 0)
                                selector--;
                            break;
                        case SDLK_DOWN:
                            if(selector < gene_keybind_max-1)
                                selector++;
                            break;
                        case SDLK_SPACE:
                        case SDLK_RETURN:
                        case SDLK_KP_ENTER:
                            waitkey = true;
                            break;
                        case SDLK_F1:
                            build_default_keybind(keybind);
                            break;
                    }
                }
            }
        }

        // Draw keybind.

        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);

        for(enum gene_keybind i=0; i<gene_keybind_max; i++) {
            sprintf(buf, "%s : %s", get_genebind_name(i), SDL_GetKeyName(keybind[i]));
            texture = loadTextureText(renderer, buf, &(dst.w), &(dst.h),
                            i == selector ? (waitkey ? color_waiting : color_selected) : color_deselected);
            dst.x = 50;
            dst.y = 50 + lineskip*i;
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
            if(i == selector) {
                // Draw hint.
                texture = loadTextureText(renderer, get_genebind_hint(i), &(dst.w), &(dst.h), color_hint);
                dst.x = SCREEN_W - dst.w - 50;
                dst.y = SCREEN_H - dst.h - 50 - lineskip;
                SDL_RenderCopy(renderer, texture, NULL, &dst);
                SDL_DestroyTexture(texture);
            }
        }
        texture = loadTextureText(renderer, local.console_resetkeybindF1, &(dst.w), &(dst.h), color_special);
        dst.x = SCREEN_W - dst.w - 50;
        dst.y = SCREEN_H - dst.h - 50;
        SDL_RenderCopy(renderer, texture, NULL, &dst);
        SDL_DestroyTexture(texture);

        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    // Clean up.
}

void build_parts_models(SDL_Renderer* renderer, struct part_model model[]) {
    for(enum part_type i=0; i<part_type_max; i++) {
        model[i].power = 0;
        model[i].speed = 0;
        model[i].hp = 0;
        switch(i) {
            case cockpit_1x1 :
                model[i].name = "tiny cockpit";
                model[i].path = path("parts", "cockpit1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.5;
                model[i].hp = 1;
                break;
            case cockpit_1x2 :
                model[i].name = "thin cockpit";
                model[i].path = path("parts", "cockpit1x2.bmp");
                model[i].w = 1;
                model[i].h = 2;
                model[i].weight = 1.5;
                model[i].hp = 2;
                break;
            case cockpit_2x1 :
                model[i].name = "wide cockpit";
                model[i].path = path("parts", "cockpit2x1.bmp");
                model[i].w = 2;
                model[i].h = 1;
                model[i].weight = 1.5;
                model[i].hp = 2;
                break;
            case cockpit_2x2 :
                model[i].name = "large cockpit";
                model[i].path = path("parts", "cockpit2x2.bmp");
                model[i].w = 2;
                model[i].h = 2;
                model[i].weight = 3.0;
                model[i].hp = 3;
                break;
            case thruster_1x1 :
                model[i].name = "little thruster";
                model[i].path = path("parts", "thruster1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.5;
                model[i].speed = 0.4;
                model[i].power = 2.0;
                break;
            case thruster_1x2 :
                model[i].name = "medium thruster";
                model[i].path = path("parts", "thruster1x2.bmp");
                model[i].w = 1;
                model[i].h = 2;
                model[i].weight = 1.0;
                model[i].speed = 1.4;
                model[i].power = 0.7;
                break;
            case thruster_2x2 :
                model[i].name = "huge thruster";
                model[i].path = path("parts", "thruster2x2.bmp");
                model[i].w = 2;
                model[i].h = 2;
                model[i].weight = 3.0;
                model[i].speed = 5.0;
                break;
            case thrustgun_1x2 :
                model[i].name = "thrustgun";
                model[i].path = path("parts", "thrustgun1x2.bmp");
                model[i].w = 1;
                model[i].h = 2;
                model[i].weight = 1.5;
                model[i].power = 1.5;
                model[i].speed = 0.6;
                break;
            case gun_1x1 :
                model[i].name = "little mushgun";
                model[i].path = path("parts", "gun1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.5;
                break;
            case leftgun_1x1 :
                model[i].name = "left mushgun";
                model[i].path = path("parts", "leftgun1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.7;
                break;
            case rightgun_1x1 :
                model[i].name = "right mushgun";
                model[i].path = path("parts", "rightgun1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.7;
                break;
            case larboardgun_1x1 : // left
                model[i].name = "larboard mushgun";
                model[i].path = path("parts", "larboardgun1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.5;
                break;
            case starboardgun_1x1 : // right
                model[i].name = "starboard mushgun";
                model[i].path = path("parts", "starboardgun1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.5;
                break;
            case reargun_1x1 :
                model[i].name = "rear mushgun";
                model[i].path = path("parts", "reargun1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 0.5;
                break;
            case triplegun_2x1 :
                model[i].name = "triple mushgun";
                model[i].path = path("parts", "triplegun2x1.bmp");
                model[i].w = 2;
                model[i].h = 1;
                model[i].weight = 2.0;
                break;
            case mushinegun_1x3 :
                model[i].name = "mushinegun";
                model[i].path = path("parts", "mushinegun1x3.bmp");
                model[i].w = 1;
                model[i].h = 3;
                model[i].weight = 3.0;
                break;
            case sniper_1x2 :
                model[i].name = "sniper";
                model[i].path = path("parts", "sniper1x2.bmp");
                model[i].w = 1;
                model[i].h = 2;
                model[i].weight = 2.0;
                break;
            case aiming_lazer_1x1:
                model[i].name = "aiming lazer";
                model[i].path = path("parts", "aiminglazer1x1.bmp");
                model[i].w = 1;
                model[i].h = 1;
                model[i].weight = 1.5;
                break;
            case lazer_1x3 :
                model[i].name = "lazer";
                model[i].path = path("parts", "lazer1x3.bmp");
                model[i].w = 1;
                model[i].h = 3;
                model[i].weight = 3.5;
                break;
            case rostrum_1x2:
                model[i].name = "rostrum";
                model[i].path = path("parts", "rostrum1x2.bmp");
                model[i].w = 1;
                model[i].h = 2;
                model[i].weight = 1.5;
                break;
            case leftspike_1x2:
                model[i].name = "left spikshield";
                model[i].path = path("parts", "leftspike1x2.bmp");
                model[i].w = 1;
                model[i].h = 2;
                model[i].weight = 1.5;
                break;
            case rightspike_1x2:
                model[i].name = "right spikshield";
                model[i].path = path("parts", "rightspike1x2.bmp");
                model[i].w = 1;
                model[i].h = 2;
                model[i].weight = 1.5;
                break;
            case part_type_max :
                break;
        }
        model[i].surface = SDL_LoadBMP(model[i].path);
        if(model[i].surface == NULL) {
            sdl_error("Unable to load part's bitmap");
            model[i].texture = NULL;
        } else {
            model[i].texture = SDL_CreateTextureFromSurface(renderer, model[i].surface);
        }
    }
}

void add_part(struct archip* archip, enum part_type part, int x, int y) {
    if(archip->nb_parts < MAX_PARTS-1 && x >= 0 && x < GRID_SIZE && y >= 0 && y < GRID_SIZE) {
        archip->part[archip->nb_parts].type = part;
        archip->part[archip->nb_parts].x = x;
        archip->part[archip->nb_parts].y = y;
        archip->nb_parts++;
    }
}

void build_archip_default(struct archip* archip) {
    archip->nb_parts = 0;
    add_part(archip, cockpit_1x2, 1, 0);
    add_part(archip, thrustgun_1x2, 0, 1);
    add_part(archip, thrustgun_1x2, 2, 1);
}

void build_archip_chapter1(struct archip* archip) {
    archip->nb_parts = 0;
    add_part(archip, cockpit_1x2, 0, 0);
    add_part(archip, thruster_1x1, 0, 2);
    add_part(archip, thruster_1x2, 1, 1);
}

void build_archip_cheated(struct archip* archip) {
    archip->nb_parts = 0;
    add_part(archip, cockpit_2x2, 3, 1);
    add_part(archip, thrustgun_1x2, 2, 1);
    add_part(archip, thrustgun_1x2, 5, 1);
    add_part(archip, triplegun_2x1, 3, 0);
    add_part(archip, leftgun_1x1, 1, 1);
    add_part(archip, rightgun_1x1, 6, 1);
    add_part(archip, leftgun_1x1, 0, 2);
    add_part(archip, rightgun_1x1, 7, 2);
    add_part(archip, thruster_1x1, 0, 3);
    add_part(archip, thruster_1x1, 7, 3);
    add_part(archip, thruster_1x1, 3, 3);
    add_part(archip, thruster_1x1, 4, 3);
    add_part(archip, thruster_1x1, 1, 2);
    add_part(archip, thruster_1x1, 6, 2);
}

void add_weapon(struct gear* gear, int x, int y, int cd, int delay, float xspeed, float yspeed, enum missile_type missile) {
    int i = gear->nb_weapon;
    if(i < MAX_GEAR-1) {
        gear->weapon[i].x = x;
        gear->weapon[i].y = y;
        gear->weapon[i].cd = cd;
        gear->weapon[i].delay = cd - delay;
        gear->weapon[i].xspeed = xspeed;
        gear->weapon[i].yspeed = yspeed;
        gear->weapon[i].missile = missile;
        gear->nb_weapon++;
    }
}

void add_thruster(struct gear* gear, int x, int y, int w, int h, SDL_Color const * const color_trail) {
    int i = gear->nb_thruster;
    if(i < MAX_GEAR-1) {
        gear->thruster[i].x = x;
        gear->thruster[i].y = y;
        gear->thruster[i].w = w;
        gear->thruster[i].h = h;
        gear->thruster[i].color_trail = color_trail;
        gear->nb_thruster++;
    }
}

void add_special_actif(struct gear* gear, int x, int y, int cd, int delay, int duration, enum special_actif_type type) {
    int i = gear->nb_special_actif;
    if(i < MAX_GEAR-1) {
        gear->special_actif[i].x = x;
        gear->special_actif[i].y = y;
        gear->special_actif[i].cd = cd;
        gear->special_actif[i].delay = delay;
        gear->special_actif[i].duration = duration;
        gear->special_actif[i].type = type;
        gear->nb_special_actif++;
    }
}

void add_special_passif(struct gear* gear, int x, int y, enum special_passif_type type) {
    int i = gear->nb_special_passif;
    if(i < MAX_GEAR-1) {
        gear->special_passif[i].x = x;
        gear->special_passif[i].y = y;
        gear->special_passif[i].type = type;
        gear->nb_special_passif++;
    }
}

void clear_grid(char grid[GRID_SIZE][GRID_SIZE], int x, int y) {
    grid[x][y] = ' ';
    if(y < GRID_SIZE-1 && grid[x][y+1] != ' ')
        clear_grid(grid, x, y+1);

    if(y > 0 && grid[x][y-1] != ' ')
        clear_grid(grid, x, y-1);

    if(x < GRID_SIZE-1 && grid[x+1][y] != ' ')
        clear_grid(grid, x+1, y);

    if(x > 0 && grid[x-1][y] != ' ')
        clear_grid(grid, x-1, y);
}

enum invalid_reason validate(struct archip* archip, struct part_model model[]) {
    char grid[GRID_SIZE][GRID_SIZE];
    int x;
    int y;
    int cockpit_ok = false;
    int thruster_ok = false;

    // Check if no cockpit or no thruster.
    for(int i=0; i<archip->nb_parts; i++) {
        if(model[archip->part[i].type].hp) {
            if(cockpit_ok) {
                return(toocockpit);
            } else {
                cockpit_ok = true;
            }
        }
        if(model[archip->part[i].type].power || model[archip->part[i].type].speed) {
            thruster_ok = true;
        }
    }
    if(!cockpit_ok) {
        return(nocockpit);
    }
    if(!thruster_ok) {
        return(nothruster);
    }

    for(int i=0; i<GRID_SIZE; i++) {
        for(int j=0; j<GRID_SIZE; j++) {
            grid[i][j] = ' ';
        }
    }

    for(int i=0; i<archip->nb_parts; i++) {
        enum part_type m = archip->part[i].type;
        x = archip->part[i].x;
        y = archip->part[i].y;
        if(x+model[m].w-1 >= GRID_SIZE || y+model[m].h-1 >= GRID_SIZE)
            return(outofgrid);
        for(int a=0; a<model[m].w; a++) {
            for(int b=0; b<model[m].h; b++) {
                if(grid[x+a][y+b] != ' ')
                    return(overlay);
                grid[x+a][y+b] = '#';
            }
        }
    }

    for(int i=0; i<archip->nb_parts; i++) {
        x = archip->part[i].x;
        y = archip->part[i].y;
        switch(archip->part[i].type) {
            case thruster_1x1:
                if(y < GRID_SIZE-1 && grid[x][y+1] != ' ')
                    return(blocked_thruster);
                break;
            case thruster_1x2:
                if(y < GRID_SIZE-2 && grid[x][y+2] != ' ')
                    return(blocked_thruster);
                break;
            case thruster_2x2:
                if(y < GRID_SIZE-2 && grid[x][y+2] != ' ' && grid[x+1][y+2] != ' ')
                    return(blocked_thruster);
                break;
            case thrustgun_1x2:
                if(y < GRID_SIZE-2 && grid[x][y+2] != ' ')
                    return(blocked_thruster);
                // break; // Let continue through blocked_gun test.
            case gun_1x1:
            case leftgun_1x1:
            case rightgun_1x1:
            case lazer_1x3:
            case mushinegun_1x3:
            case sniper_1x2:
                if(y > 0 && grid[x][y-1] != ' ')
                    return(blocked_gun);
                break;
            case starboardgun_1x1:
                if(x < GRID_SIZE-1 && grid[x+1][y] != ' ')
                    return(blocked_gun);
                break;
            case larboardgun_1x1:
                if(x > 0 && grid[x-1][y] != ' ')
                    return(blocked_gun);
                break;
            case triplegun_2x1:
                if(y > 0 && ( grid[x][y-1] != ' ' || grid[x+1][y-1] != ' ') )
                    return(blocked_gun);
                break;
            case reargun_1x1:
                if(y < GRID_SIZE-1 && grid[x][y+1] != ' ')
                    return(blocked_gun);
                break;
            case rostrum_1x2:
                if(y > 0) for(int i=0; i<GRID_SIZE; i++) {
                    if(grid[i][y-1] != ' ')
                        return(rostrum_not_forward);
                }
                break;
            case leftspike_1x2:
                if(x > 0) for(int i=0; i<GRID_SIZE; i++) {
                    if(grid[x-1][i] != ' ')
                        return(spikeshied_not_extern);
                }
                break;
            case rightspike_1x2:
                if(x < GRID_SIZE-1) for(int i=0; i<GRID_SIZE; i++) {
                    if(grid[x+1][i] != ' ')
                        return(spikeshied_not_extern);
                }
                break;
            case aiming_lazer_1x1:
            case cockpit_1x1:
            case cockpit_1x2:
            case cockpit_2x1:
            case cockpit_2x2:
            case part_type_max:
                break;
        }
    }

    // Check connexity.
    for(int i=0; i<GRID_SIZE; i++) {
        for(int j=0; j<GRID_SIZE; j++) {
            if(grid[i][j] != ' ') {
                x = i;
                y = j;
                break;
            }
        }
    }
    clear_grid(grid, x, y);
    for(int i=0; i<GRID_SIZE; i++) {
        for(int j=0; j<GRID_SIZE; j++) {
            if(grid[i][j] != ' ') {
                return(nonconnexe);
            }
        }
    }

    // Find top-left corner.
    int top = GRID_SIZE;
    int lft = GRID_SIZE;
    for(int i=0; i<archip->nb_parts; i++) {
        if(archip->part[i].x < lft)
            lft = archip->part[i].x;
        if(archip->part[i].y < top)
            top = archip->part[i].y;
    }

    // Move to the top-left corner.
    for(int i=0; i<archip->nb_parts; i++) {
        archip->part[i].x -= lft;
        archip->part[i].y -= top;
    }

    return(valid);
}

SDL_Surface* build_archip(SDL_Renderer* renderer, struct gear* gear, int chapter) {
    SDL_Texture* title;
    SDL_Texture* hint;
    SDL_Texture* cursor;
    SDL_Surface* return_surface;
    SDL_Rect title_dst;
    SDL_Rect hint_dst;
    SDL_Keycode keybind[gene_keybind_max];
    SDL_Rect dst;
    SDL_Texture* texture;
    char buf[BUFSIZ];
    char* message = NULL;
    struct archip archip;
    static struct archip saved_archip;
    static int first_run = true;
    struct part_model model[part_type_max];
    int stop;
    int cancel;
    int double_valid;
    int double_escape;
    int selector;
    enum invalid_reason invalid_reason = 0;
    float weight;
    float power;

    // Load textures.

    title = loadTextureSize(renderer, path_localized("text", "banner_archip.bmp"), &(title_dst.w), &(title_dst.h));
    if(title == NULL)
        return(NULL);
    title_dst.x = SCREEN_W/2 - title_dst.w/2;
    title_dst.y = 30;
    hint = loadTextureText(renderer, local.console_buildyourship, &(hint_dst.w), &(hint_dst.h), color_hint);
    if(hint == NULL)
        return(NULL);
    hint_dst.w *= 1.5;
    hint_dst.h *= 1.5;
    hint_dst.x = SCREEN_W/2 - hint_dst.w/2;
    hint_dst.y = title_dst.y + title_dst.h + lineskip;
    cursor = loadTexture(renderer, path("", "cursor.bmp"));
    if(cursor == NULL)
        return(NULL);

    build_parts_models(renderer, model);

    // Check parts loading.
    for(int i=0; i<part_type_max; i++) {
        if(model[i].texture == NULL) {
            return(NULL);
        }
    }

    // Init.

    archip.nb_parts = 0;
    selector = 0;
    cancel = false;
    double_valid = false;
    double_escape = false;
    if(chapter == 1) {
        stop = true;
    } else {
        stop = false;
    }
    if(first_run) {
        first_run = false;
        build_archip_default(&archip);
        saved_archip = archip;
    } else {
        archip = saved_archip;
    }
    build_default_keybind(keybind);

    // LOOP

    while(!stop) {
        SDL_Event event;
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                    cancel = true;
                    stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                message = NULL;
                if(event.key.keysym.sym == SDLK_ESCAPE) { // Cancel.
                    if(double_escape) {
                        cancel = true;
                        stop = true;
                        break;
                    } else {
                        double_escape = true;
                    }
                } else if(event.key.keysym.sym == SDLK_F1) {
                    gene_keybind_menu(renderer, keybind);
                } else if(event.key.keysym.sym == SDLK_F2) {
                    build_archip_cheated(&archip);
                } else {
                    for(enum gene_keybind i=0; i<gene_keybind_max; i++) {
                        if(keybind[i] == event.key.keysym.sym) {
                            switch(i) {
                                case gene_scrollup:
                                    if(selector > 0) {
                                        selector--;
                                    }
                                    break;
                                case gene_scrolldown:
                                    if(selector < archip.nb_parts-1) {
                                        selector++;
                                    }
                                    break;
                                case gene_raiseup:
                                    if(archip.nb_parts >= 2 && selector >= 1) {
                                        struct part part = archip.part[selector];
                                        archip.part[selector] = archip.part[selector-1];
                                        selector--;
                                        archip.part[selector] = part;
                                    }
                                    break;
                                case gene_pushdown:
                                    if(archip.nb_parts >= 2 && selector <= archip.nb_parts-2) {
                                        struct part part = archip.part[selector];
                                        archip.part[selector] = archip.part[selector+1];
                                        selector++;
                                        archip.part[selector] = part;
                                    }
                                    break;
                                case gene_changenext:
                                    if(archip.nb_parts > 0) {
                                        // archip.part[selector].type = (archip.part[selector].type+1) % max_part_type;
                                        archip.part[selector].type++;
                                        if(archip.part[selector].type >= part_type_max)
                                            archip.part[selector].type = 0;
                                    }
                                    break;
                                case gene_changeprev:
                                    if(archip.nb_parts > 0) {
                                        if(archip.part[selector].type > 0)
                                            archip.part[selector].type--;
                                        else
                                            archip.part[selector].type = part_type_max-1;
                                    }
                                    break;
                                case gene_add:
                                    if(archip.nb_parts < MAX_PARTS-1) {
                                        archip.part[archip.nb_parts].type = 0; // First element by default.
                                        archip.part[archip.nb_parts].x = 0;
                                        archip.part[archip.nb_parts].y = 0;
                                        selector = archip.nb_parts;
                                        archip.nb_parts++;
                                    }
                                    break;
                                case gene_del:
                                    if(archip.nb_parts > 0) {
                                        for(int i=selector; i<archip.nb_parts-1; i++) {
                                            archip.part[i] = archip.part[i+1];
                                        }
                                        archip.nb_parts--;
                                        if(archip.nb_parts > 0 && selector >= archip.nb_parts) {
                                            selector--;
                                        }
                                    }
                                    break;
                                case gene_moveup:
                                    if(archip.nb_parts > 0 && archip.part[selector].y > 0) {
                                        archip.part[selector].y--;
                                    }
                                    break;
                                case gene_movedown:
                                    if(archip.nb_parts > 0 && archip.part[selector].y < GRID_SIZE-1) {
                                        archip.part[selector].y++;
                                    }
                                    break;
                                case gene_moveleft:
                                    if(archip.nb_parts > 0 && archip.part[selector].x > 0) {
                                        archip.part[selector].x--;
                                    }
                                    break;
                                case gene_moveright:
                                    if(archip.nb_parts > 0 && archip.part[selector].x < GRID_SIZE-1) {
                                        archip.part[selector].x++;
                                    }
                                    break;
                                case gene_validate:
                                    invalid_reason = validate(&archip, model);
                                    if(invalid_reason == valid) {
                                        if(double_valid) {
                                            stop = true;
                                        } else {
                                            double_valid = true;
                                        }
                                    }
                                    break;
                                case gene_keybind_max:
                                    break;
                            } // End of switch
                            break;
                        } // End of if(keybind == keysym)
                    } // End of for(i=0:gene_keybind_max)
                }
            } // End of if(event.type == SDL_KEYDOWN)
            if(event.type == SDL_KEYDOWN && event.key.keysym.sym != SDLK_RETURN && event.key.keysym.sym != SDLK_KP_ENTER && event.key.keysym.sym != SDLK_SPACE) {
                double_valid = false;
            }
            if(event.type == SDL_KEYDOWN && event.key.keysym.sym != SDLK_ESCAPE) {
                double_escape = false;
            }
        }

        // Clear the screen.
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);

        // Draw banner and hint.
        SDL_RenderCopy(renderer, title, NULL, &title_dst);
        SDL_RenderCopy(renderer, hint, NULL, &hint_dst);

        // Draw the parts' list.
        for(int i=0; i<archip.nb_parts; i++) {
            texture = loadTextureText(renderer, model[archip.part[i].type].name, &(dst.w), &(dst.h),
                                      i == selector ? color_selected : color_deselected);
            dst.x = 30;
            dst.y = hint_dst.y + hint_dst.h + lineskip * (i+1);
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
        }

        // Draw the grid.
        SDL_SetRenderDrawColor(renderer, 64, 64, 64, 255); // Light grey.
        for(int x=0; x<=GRID_SIZE; x++) {
            SDL_RenderDrawLine(renderer,
                GRID_MULTIPL + GRID_X_OFFSET + x*(GRID_CASE_SIZE-GRID_MULTIPL),
                GRID_MULTIPL + GRID_Y_OFFSET,
                GRID_MULTIPL + GRID_X_OFFSET + x*(GRID_CASE_SIZE-GRID_MULTIPL),
                GRID_MULTIPL + GRID_Y_OFFSET + GRID_SIZE*GRID_CASE_SIZE
            );
        }
        for(int y=0; y<=GRID_SIZE; y++) {
            SDL_RenderDrawLine(renderer,
                GRID_MULTIPL + GRID_X_OFFSET,
                GRID_MULTIPL + GRID_Y_OFFSET + y*GRID_CASE_SIZE,
                GRID_MULTIPL + GRID_X_OFFSET + GRID_SIZE*(GRID_CASE_SIZE-GRID_MULTIPL),
                GRID_MULTIPL + GRID_Y_OFFSET + y*GRID_CASE_SIZE
            );
        }

        // Draw the parts.
        for(int i=archip.nb_parts-1; i>=0; i--) {
            dst.x = GRID_X_OFFSET + archip.part[i].x*(GRID_CASE_SIZE-GRID_MULTIPL);
            dst.y = GRID_Y_OFFSET + archip.part[i].y*GRID_CASE_SIZE;
            dst.w = model[archip.part[i].type].surface->w * GRID_MULTIPL;
            dst.h = model[archip.part[i].type].surface->h * GRID_MULTIPL;

            SDL_RenderCopy(renderer, model[archip.part[i].type].texture, NULL, &dst);
        }

        // Draw the cursor.
        if(archip.nb_parts > 0) {
            dst.x = GRID_X_OFFSET + archip.part[selector].x*(GRID_CASE_SIZE-GRID_MULTIPL);
            dst.y = GRID_Y_OFFSET + archip.part[selector].y*GRID_CASE_SIZE;
            dst.w = model[archip.part[selector].type].surface->w * GRID_MULTIPL;
            dst.h = model[archip.part[selector].type].surface->h * GRID_MULTIPL;
            SDL_RenderCopy(renderer, cursor, NULL, &dst);
        }

        // Draw controls.
        for(enum gene_keybind i=0; i<gene_keybind_max; i++) {
            sprintf(buf, "%s : %s", get_genebind_name(i), SDL_GetKeyName(keybind[i]));
            texture = loadTextureText(renderer, buf, &(dst.w), &(dst.h), i == gene_validate ? color_special : color_hint);
            dst.x = 10 + GRID_X_OFFSET + GRID_CASE_SIZE*GRID_SIZE;
            dst.y = hint_dst.y + hint_dst.h + lineskip + i*lineskip;
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
        }
        texture = loadTextureText(renderer, local.console_changekeybindF1, &(dst.w), &(dst.h), color_special);
        dst.x = 10 + GRID_X_OFFSET + GRID_CASE_SIZE*GRID_SIZE;
        dst.y = hint_dst.y + hint_dst.h + lineskip + gene_keybind_max*lineskip;
        SDL_RenderCopy(renderer, texture, NULL, &dst);
        SDL_DestroyTexture(texture);

        // Draw validate error message.
        message = NULL;
        if(double_valid && invalid_reason == valid) {
            message = local.console_doublevalid;
        } else if(double_escape) {
            message = local.console_doubleescape;
        } else if(invalid_reason != valid) {
            message = local.console_invalidmsg[invalid_reason];
        }
        if(message != NULL) {
            texture = loadTextureText(renderer, message, &(dst.w), &(dst.h),
                double_valid ? color_hint : color_warning);
            dst.x = SCREEN_W/2 - dst.w/2;
            dst.y = SCREEN_H - 50;
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
        }

        // Update HP, weight and thrusters.

        weight = 0.0;
        power = 0.0;
        gear->speed = 0.0;
        gear->delta_v = 0;
        gear->hp = 0;
        int collision_power = 0;
        for(int i=0; i<archip.nb_parts; i++) {
            weight += model[archip.part[i].type].weight;
            power += model[archip.part[i].type].power;
            gear->speed += model[archip.part[i].type].speed;
            gear->hp += model[archip.part[i].type].hp;
            if(archip.part[i].type == rostrum_1x2) {
                collision_power += RAM_POWER;
            } else if(archip.part[i].type == leftspike_1x2 || archip.part[i].type == rightspike_1x2) {
                collision_power += SPIKE_POWER;
            }
        }
        if(weight != 0) {
            gear->delta_v = power / weight;
            // gear->speed = gear->speed / weight;
        } else {
            gear->delta_v = 0;
            gear->speed = 0;
        }

        // Display HP, weight and thrusters.

        sprintf(buf, "[%s: %.1f] [%s: %.1f] [%s: %.1f] [%s: %.1f]",
                local.console_weight, weight,
                local.console_power, power,
                local.console_agility, gear->delta_v,
                local.console_speed, gear->speed);
        texture = loadTextureText(renderer, buf, &(dst.w), &(dst.h), color_hint);
        dst.x = GRID_X_OFFSET;
        dst.y = GRID_Y_OFFSET + GRID_CASE_SIZE*GRID_SIZE + 50;
        SDL_RenderCopy(renderer, texture, NULL, &dst);
        SDL_DestroyTexture(texture);

        sprintf(buf, "[%s: %d] [%s: %d]",
                local.console_shield, gear->hp,
                local.console_ram, collision_power);
        texture = loadTextureText(renderer, buf, &(dst.w), &(dst.h), color_hint);
        dst.x = GRID_X_OFFSET;
        dst.y = GRID_Y_OFFSET + GRID_CASE_SIZE*GRID_SIZE + 50 + lineskip;
        SDL_RenderCopy(renderer, texture, NULL, &dst);
        SDL_DestroyTexture(texture);

        // Display!!

        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    if(chapter == 1) {
        build_archip_chapter1(&archip);
    } else if(cancel) {
        build_archip_default(&archip);
    } else {
        saved_archip = archip;
    }

    // Build ship's Surface and Gear.

    int bot = 0;
    int rgt = 0;
    gear->nb_thruster = 0;
    gear->nb_weapon = 0;
    gear->nb_special_actif = 0;
    gear->nb_special_passif = 0;

    for(int i=0; i<archip.nb_parts; i++) {
        int w = archip.part[i].x * PART_SIZE + model[archip.part[i].type].surface->w;
        int h = archip.part[i].y * PART_SIZE + model[archip.part[i].type].surface->h;
        if(w > rgt)
            rgt = w;
        if(h > bot)
            bot = h;
    }

    if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
        return_surface = SDL_CreateRGBSurface(0, rgt, bot, 32, 0xFF000000, 0x00FF0000, 0x0000FF00, 0x000000FF);
    } else {
        return_surface = SDL_CreateRGBSurface(0, rgt, bot, 32, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000);
    }
    if(return_surface == NULL) {
        sdl_error("Unable to create Alhpa surface");
        return_surface = SDL_CreateRGBSurface(0, rgt, bot, 32, 0, 0, 0, 0);
    }

    for(int i=archip.nb_parts-1; i>=0; i--) {
        // Add part to the surface.
        SDL_Rect dst;
        dst.w = model[archip.part[i].type].surface->w;
        dst.h = model[archip.part[i].type].surface->h;
        dst.x = archip.part[i].x * (PART_SIZE-1);
        dst.y = archip.part[i].y * PART_SIZE;
        SDL_BlitSurface(model[archip.part[i].type].surface, NULL, return_surface, &dst);

        dst.x = dst.x + (PART_SIZE+1)/2 - rgt/2;
        dst.y = dst.y + (PART_SIZE+1)/2 - bot/2;
        // Now dst.x and dst.y are coords of the center of the 1x1 part relative to the center of the ship.

        // Add thruster and weapon to gear.
        switch(archip.part[i].type) {
            case thruster_1x1:
                add_thruster(gear, dst.x, dst.y, 4, 25, color_mushtrail);
                break;
            case thruster_1x2:
                add_thruster(gear, dst.x, dst.y + PART_SIZE, 6, 30, color_mushtrail);
                break;
            case thruster_2x2:
                add_thruster(gear, dst.x + PART_SIZE/2, dst.y + PART_SIZE/2, 15, 40, color_mushtrail);
                break;
            case gun_1x1:
                add_weapon(gear, dst.x, dst.y, 30, 0, 0.0, -5.0, missile_mushbullet);
                break;
            case leftgun_1x1:
                add_weapon(gear, dst.x, dst.y, 30, 0, -1.0, -4.0, missile_mushbullet);
                break;
            case rightgun_1x1:
                add_weapon(gear, dst.x, dst.y, 30, 0, 1.0, -4.0, missile_mushbullet);
                break;
            case thrustgun_1x2:
                add_weapon(gear, dst.x, dst.y, 30, 15, 0.0, -5.0, missile_mushile);
                add_thruster(gear, dst.x, dst.y + PART_SIZE, 6, 30, color_mushtrail);
                break;
            case reargun_1x1:
                add_weapon(gear, dst.x, dst.y, 30, 0, 0.0, 5.0, missile_mushbullet);
                break;
            case larboardgun_1x1:
                add_weapon(gear, dst.x, dst.y, 30, 0, -5.0, 0.0, missile_mushbullet);
                break;
            case starboardgun_1x1:
                add_weapon(gear, dst.x, dst.y, 30, 0, 5.0, 0.0, missile_mushbullet);
                break;
            case triplegun_2x1:
                add_weapon(gear, dst.x, dst.y, 30, 0, -1.0, -4.0, missile_mushbullet);
                add_weapon(gear, dst.x + PART_SIZE/2, dst.y, 30, 0, 0.0, -5.0, missile_mushbullet);
                add_weapon(gear, dst.x + PART_SIZE, dst.y, 30, 0, 1.0, -4.0, missile_mushbullet);
                break;
            case sniper_1x2:
                add_weapon(gear, dst.x, dst.y, 100, 0, 0, -15.0, missile_sniper);
                break;
            case mushinegun_1x3:
                add_weapon(gear, dst.x, dst.y, 150, 0, 0.0, -5.0, missile_mushile);
                add_weapon(gear, dst.x, dst.y, 150, 10, 0.0, -5.0, missile_mushile);
                add_weapon(gear, dst.x, dst.y, 150, 20, 0.0, -5.0, missile_mushile);
                add_weapon(gear, dst.x, dst.y, 150, 30, 0.0, -5.0, missile_mushile);
                add_weapon(gear, dst.x, dst.y, 150, 40, 0.0, -5.0, missile_mushile);
                add_weapon(gear, dst.x, dst.y, 150, 50, 0.0, -5.0, missile_mushile);
                break;
            case lazer_1x3:
                add_special_actif(gear, dst.x, dst.y, 300, 20, 100, special_actif_beamlazer);
                break;
            case aiming_lazer_1x1:
                add_special_passif(gear, dst.x, dst.y, special_passif_aiminglazer);
                break;
            case rostrum_1x2:
                add_special_passif(gear, dst.x, dst.y, special_passif_rostrum);
                break;
            case leftspike_1x2:
            case rightspike_1x2:
                add_special_passif(gear, dst.x, dst.y, special_passif_spikedshield);
                break;
            case cockpit_1x1:
            case cockpit_1x2:
            case cockpit_2x1:
            case cockpit_2x2:
            case part_type_max:
                break;
        }
    }

    // Calculate speed.

    weight = 0.0;
    power = 0.0;
    gear->speed = 0.0;
    gear->hp = 0;
    for(int i=0; i<archip.nb_parts; i++) {
        weight += model[archip.part[i].type].weight;
        power += model[archip.part[i].type].power;
        gear->speed += model[archip.part[i].type].speed;
        gear->hp += model[archip.part[i].type].hp;
    }
        if(weight != 0) {
            gear->delta_v = power / weight;
            // gear->speed = gear->speed / weight;
        } else {
            gear->delta_v = 0;
            gear->speed = 0;
        }

    // Clean up.
    SDL_DestroyTexture(title);
    SDL_DestroyTexture(cursor);
    for(int i=0; i<part_type_max; i++) {
        SDL_FreeSurface(model[i].surface);
        SDL_DestroyTexture(model[i].texture);
    }

    return(return_surface);
}
