
/* Copyright  2014 Pierre Vidal <p.terkal@gmail.com>
 * This program is free software. It comes without no warranty.
 * You can redistribute it and/or modify it under the terms of the
 * Do What The Fuck You Want To Public License, Version 2, as published by Sam Hocevar.
 * See the COPYING file or http://www.wtfpl.net/ for more details. */

#include "main.h"

void chapter_01(SDL_Renderer* renderer);

void cinematic(SDL_Renderer* renderer, int chapter) {

    switch(chapter) {
        case 1: chapter_01(renderer); break;
    }
}

void chapter_01(SDL_Renderer* renderer) {
    const int txt_len = 4;
    const int txt_duration = 300;
    SDL_Texture* ship;
    SDL_Texture* sun;
    SDL_Rect ship_dst, sun_dst;
    SDL_Event event;
    const int postend = 300;
    const int duration = txt_len*txt_duration + postend;
    int nb_stars = rand()%30 + 70;
    struct star stars[nb_stars];
    char* txt[txt_len];
    SDL_Texture* text_ure[txt_len];
    SDL_Rect txt_dst[txt_len];

    if(load_cinematic_text(1, txt, txt_len) == EXIT_FAILURE) {
        return;
    }

    for(int i=0; i<txt_len; i++) {
        text_ure[i] = loadTextureText(renderer, txt[i], &(txt_dst[i].w), &(txt_dst[i].h), color_muship);
        txt_dst[i].x = 100;
        txt_dst[i].y = 100;
    }

    generate_stars(stars, nb_stars);

    // Load textures.
    ship = loadTextureSize(renderer, path("scenario", "ship.bmp"), &(ship_dst.w), &(ship_dst.h));
    sun  = loadTextureSize(renderer, path("scenario", "sun.bmp"), &(sun_dst.w), &(sun_dst.h));
    if(ship == NULL || sun == NULL) {
        return;
    }

    // Sun won't move.
    sun_dst.x = SCREEN_W/2 - sun_dst.w/2;
    sun_dst.y = SCREEN_H/2 - sun_dst.h/2;

    for(int i=0; i<duration; i++) {

        // Check for exit.
        SDL_PollEvent(&event);
        if(event.type == SDL_KEYDOWN || event.type == SDL_QUIT)
            break;

        // Draw space and stars.
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);
        draw_stars(renderer, stars, nb_stars);

        // Draw the fading sun.
        if(duration-i < 256 + postend) {
            if(duration - i < postend) {
                SDL_SetTextureColorMod(sun, 0, 0, 0);
            } else {
                SDL_SetTextureColorMod(sun, duration-i-postend, duration-i-postend, duration-i-postend);
            }
        }
        SDL_RenderCopy(renderer, sun, NULL, &sun_dst);

        // Draw the ship sailing.
        ship_dst.x = SCREEN_W/2 - ship_dst.w/2 + 70 + i/10;
        ship_dst.y = SCREEN_H/2 - ship_dst.h/2 + 80 + abs(i%200 - 100)/8 + i/20;
        SDL_RenderCopy(renderer, ship, NULL, &ship_dst);

        // Draw ship's talk.
        if(i/txt_duration < txt_len) {
            SDL_RenderCopy(renderer, text_ure[i/txt_duration], NULL, &(txt_dst[i/txt_duration]));
        }

        // Present and wait.
        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    for(int i=0; i<txt_len; i++) {
        SDL_DestroyTexture(text_ure[i]);
    }
    SDL_DestroyTexture(ship);
    SDL_DestroyTexture(sun);
    unload_cinematic_text(txt, txt_len);
}

void briefing(SDL_Renderer* renderer, int chapter) {
    int nb_stars = rand()%30 + 70;
    struct star star[nb_stars];
    SDL_Rect dst;
    char* txt[MAX_BRIEFING];

    load_briefing(txt, chapter);

    // Generate stars.
    for(int i=0; i<nb_stars; i++) {
        star[i].x = rand()%SCREEN_W;
        star[i].y = rand()%SCREEN_H;
        star[i].r = rand()%256;
        star[i].g = rand()%256;
        star[i].b = rand()%256;
    }

    for(int i=0; i<nb_stars; i++) {
        SDL_SetRenderDrawColor(renderer, star[i].r, star[i].g, star[i].b, 255);
        SDL_RenderDrawPoint(renderer, star[i].x, star[i].y);
    }
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
    SDL_RenderClear(renderer);

    for(int i=0; txt[i][0] != '\0'; i++) {
        SDL_Texture* texture = loadTextureText(renderer, txt[i], &(dst.w), &(dst.h), color_muship);
        dst.x = SCREEN_W/2 - dst.w/2;
        dst.y = SCREEN_H/4 + lineskip*i;
        SDL_RenderCopy(renderer, texture, NULL, &dst);
        SDL_DestroyTexture(texture);
    }

    int stop = false;
    // for(int i=0; !stop && i<1000; i++) {
    while(!stop) {
        SDL_Event event;
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_KEYDOWN || event.type == SDL_QUIT) {
                stop = true;
            }
        }
        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    unload_briefing(txt);
}
