
/* Copyright  2014 Pierre Vidal <p.terkal@gmail.com>
 * This program is free software. It comes without no warranty.
 * You can redistribute it and/or modify it under the terms of the
 * Do What The Fuck You Want To Public License, Version 2, as published by Sam Hocevar.
 * See the COPYING file or http://www.wtfpl.net/ for more details. */

#ifndef MAIN_H
#define MAIN_H

#include <SDL2/SDL.h>
#include <stdlib.h>

#define VERSION "Version Beta 2.3.2"
#define INIT_FULLSCREEN true
#define ARCADE_LEVEL 0
#define DEFAULT_LANGUAGE "EN"

/* * * MAIN GOALS * * *
 * infinite random level
 * * * * * * * * * * * */

/* * * FIXME * * *
 * ingame special weapon reload icon
 * lazer delay is too fuzy :/: draw lazer's charging halo
 * add color parameter to lazer and aiming lazer.
 * auto-aiming bullets don't take into account target's speed
 * changing language doesn't change already loaded menu's textures
 * "more than 2 keys" diagonale move bug
 * modify chapter1's cinematic accordingly to the new initial ship design
 * strange fullscreen bug
 * cinematic text must fade in/out
 * less bmp files
 * graphical error feedback
 * * * * * * * * */

/* * * IDEAS * * *
 * ship galery + gear description
 * more visual distinction between mech gears (more thruster color, more missile's aspect, ...)
 * drag'n'drop archip design
 * acceleration (power/weight)
 * mechip weight and acceleration
 * missiles phases : blinking mines, bitting piranha-like missiles, ...
 * icon for explicit function of muship's parts in genetic console
 * how to make ramming more fun? more easy?
 * energy with max gauge and income rate: perma-fire won't be a good choice
 * // SPECIAL WEAPONS // *
 * mushotgun
 * big mushinegun (with alt-file key)
 * activable speed-thruster ("booster")
 * more diagonal guns
 * mush guided or aiming missile
 * forward bomb cloud
 * energy shield (+1 "life" or activable invincibility)
 * replace cooldown by finite charges
 * forward mush mine
 * more special weapons ideas: bombs, bots, aiming missiles, guided missile, missile cloud, missile ring, black-hole, ...
 * // ENEMY // *
 * mechip with no forward weapon, but spreaded side-forward guns, and coordinated left/right guns
 * // STORY // *
 * unique level-end bosses
 * pursuit special level(s) -> reversed gameplay
 * allow inter-level choices
 * multiple campaigns
 * build points, parts' unlocks (tree? only campaign or even arcade unlocks?)
 * first level with great muship and huge waves, so-many-lifes, false "lose" against a boss mechip
 * asteroids in run def', ships-asteroid and missile-asteroid collisions
 * asteroids : diagonal path, random speed, but same direction for a same wave
 * make particles quantity increase for saturn's ring level and other levels in space clouds. (150 seems to be fun)
 * // SOUND // *
 * music menu theme
 * muship special explosion
 * missiles fire?
 * // SCRIPTS // *
 * make particles colors and global quantity parametrable in level file
 * add file for missiles and add color_trail parameter for thrusters and missiles
 * // KEYBIND // *
 * disable same key on different actions
 * enable same action on different keys
 * one key-action per special gear
 * make F11 recognized in every menu.
 * // FINE GRAIN // *
 * scalable difficulty
 * save archip as image file
 * load and save archip in files
 * better hit-box (ellipse for the muship)
 * quadtree collision detection
 * destroyable missiles
 * more and random explosions
 * * * *  * * * */

/* * * Big Ideas * * *
 * Autonomous drones (still customizables)
 * multiplayer (coop/pvp)
 * Core classes (drone, hunter, cruiser, destroyer, station, ...)
 * colorshemes (with gameplay impact?)
 * * * * * * * * * * */

#define true 1
#define false 0
#define SCREEN_W 800
#define SCREEN_H 600
// #define SCREEN_W 1024
// #define SCREEN_H 748
#define MAX_PARTS       16
#define MAX_GEAR        MAX_PARTS * 6 // The mushinegun has 6 shots in one cooldown cycle
#define MAX_WAVE        32
#define MAX_DIALOGUE    16
#define MAX_SHIP_MODEL  32
#define MAX_BRIEFING    16
#define RAM_POWER       3
#define SPIKE_POWER     1
#define MAX_KEYBIND_HINT 3

#if ( defined _WIN32 || defined _WIN64 || defined __WIN32__ || defined __WINDOWS__ )
# define SHMUP_WINDOWS
#endif

#ifndef BUFSIZ
# define BUFSIZ 1024
#endif

static const SDL_Color color_green = {0, 255, 0, 255}; // RGB-A Green
static const SDL_Color color_violet = {255, 0, 255, 255}; // RGB-A Violet
static const SDL_Color color_purple = {128, 0, 128, 255}; // RGB-A Purple
static const SDL_Color color_orange = {255, 128, 0, 255}; // RGB-A Orange
static const SDL_Color color_blue = {0, 255, 255, 255}; // RGB-A Blue (Cyan, actually)
static const SDL_Color color_red = {255, 0, 0, 255}; // RGB-A Red

static SDL_Color const * const color_deselected = &color_green;
static SDL_Color const * const color_selected = &color_violet;
static SDL_Color const * const color_waiting = &color_orange;

static SDL_Color const * const color_true = &color_green;
static SDL_Color const * const color_false = &color_orange;

static SDL_Color const * const color_muship = &color_green;
static SDL_Color const * const color_mech = &color_orange;
static SDL_Color const * const color_player = &color_blue;
static SDL_Color const * const color_nhy = &color_violet;

static SDL_Color const * const color_hint = &color_green;
static SDL_Color const * const color_special = &color_blue;
static SDL_Color const * const color_warning = &color_orange;

static SDL_Color const * const color_evil = &color_red;

static SDL_Color const * const color_helltrail = &color_red;
static SDL_Color const * const color_mushtrail = &color_blue;
static SDL_Color const * const color_darkmushtrail = &color_purple;

static char const * const lang_code[] = {
    "EN",
    "FR",
    ""
};

enum game_keybind {
    game_keybind_moveup,
    game_keybind_movedown,
    game_keybind_moveleft,
    game_keybind_moveright,
    game_keybind_fire,
    game_keybind_altfire,
    game_keybind_max
};

enum gene_keybind {
    gene_validate,
    gene_moveup,
    gene_movedown,
    gene_moveleft,
    gene_moveright,
    gene_scrollup,
    gene_scrolldown,
    gene_raiseup,
    gene_pushdown,
    gene_add,
    gene_del,
    gene_changenext,
    gene_changeprev,
    gene_keybind_max
};

enum invalid_reason {
    valid,
    overlay,
    nothruster,
    nocockpit,
    toocockpit,
    outofgrid,
    blocked_thruster,
    blocked_gun,
    rostrum_not_forward,
    spikeshied_not_extern,
    nonconnexe,
    invalid_reason_max
};

enum score_entry {
    score_dodge,
    score_destroy,
    score_shot,
    score_hit,
    score_time,
    score_entry_max
};

enum display_entry_id {
    display_entry_return,
    display_entry_fullscreen,
    display_entry_trail,
    display_entry_explosion,
    display_entry_particles,
    display_entry_max
};

enum option_entry_id {
    option_entry_return,
    option_entry_controls,
    option_entry_display,
    option_entry_sound,
    option_entry_language,
    option_entry_max
};

enum sound_entry_id {
    sound_entry_return,
    sound_entry_mute,
    sound_entry_volume,
    sound_entry_max
};

enum mech_gear_type {
    mech_gear_helltorpedo,
    mech_gear_minelayer,
    mech_gear_aimingturret,
    mech_gear_lazer,
    mech_gear_leftgun,
    mech_gear_rightgun,
    mech_gear_hellthruster,
    mech_gear_hellreactor
};

struct score {
    int nb_dodge;
    int nb_destroy;
    int nb_shot;
    int nb_hit;
    int time; // number of frame
};

struct star {
    int x;
    int y;
    int r; // Red.
    int g; // Green.
    int b; // Blue.
};

enum missile_type {
    missile_mechile,
    missile_mushile,
    missile_mushbullet,
    missile_mechbullet,
    missile_mechautobullet,
    missile_mushmine,
    missile_mechmine,
    missile_sniper,
    missile_torpedo,
    missile_type_max
};

enum special_actif_type {
    special_actif_beamlazer
    // special_actif_ringlazer
    // special_actif_emergencyblink
    // special_actif_shadowmode
    // special_actif_booster
};

enum special_passif_type {
    special_passif_aiminglazer,
    special_passif_rostrum, // ramming rostrum
    special_passif_spikedshield
    // special_passif_bioreactor
};

struct weapon {
    int x;
    int y;
    int cd;
    int delay;
    float xspeed;
    float yspeed;
    enum missile_type missile;
};

struct thruster {
    int x;
    int y;
    int w; // Thickness
    int h; // Length
    SDL_Color const * color_trail;
};

struct special_actif {
    int x;
    int y;
    int cd;
    int delay;
    int duration;
    enum special_actif_type type;
};

struct special_passif {
    int x;
    int y;
    enum special_passif_type type;
};

struct gear {
    int nb_weapon;
    struct weapon weapon[MAX_GEAR];
    int nb_thruster;
    struct thruster thruster[MAX_GEAR];
    int nb_special_actif;
    struct special_actif special_actif[MAX_GEAR];
    int nb_special_passif;
    struct special_passif special_passif[MAX_GEAR];
    float speed; // Delta-Y
    float delta_v;
    int hp;
};

struct ship_model {
    char* name;
    SDL_Texture* texture;
    struct gear gear;
};

struct optimizations {
    // Booleans. If true, draw it. Else, don't.
    int explosion;
    int trails;
    int particles;
};

enum speaker {
    speaker_muship,
    speaker_player,
    speaker_mech,
    speaker_evil,
    speaker_nhy
};

struct dialogue {
    char* text;
    int timestart;
    int duration;
    SDL_Texture* texture;
    int w;
    int h;
    int y_shift;
    enum speaker speaker;
};

struct wave {
    int model_id;
    float timestart; // Number of frame since the begining of the run before wave start.
    float duration;  // Number of frame since the begining of the wave before wave end.
    int spawn_per_sec; // Each 100 frames.
    float accumulator; // Private : used for internal calculation.
};

struct run {
    int nb_waves;
    struct wave waves[MAX_WAVE];
    int nb_dialogue;
    struct dialogue dialogues[MAX_DIALOGUE];
};

enum sound_id {
    sound_explosion,
    sound_explosion_grave,
    sound_thruster_doppler,
/*
    sound_thruster_mush,
    sound_thruster_mech,
*/
    sound_id_max
};

struct local {
    char code[2+1];
    char* optionmenu_entry[option_entry_max];
    char* storymenu_returntomain;
    char* keybindmenu_hints[MAX_KEYBIND_HINT];
    char* keybindmenu_entry[game_keybind_max];
    char* soundmenu_entry[sound_entry_max];
    char* optimenu_entry[display_entry_max];
    char* optimenu_enable;
    char* optimenu_disable;
    char* gameover_pressenter;
    char* gameover_scores[score_entry_max];
    char* gameover_accuracy;
    char* gameover_ult_geneticefficiency;
    char* gameover_geneticefficiency;
    char* gameover_ult_mushroomavenger;
    char* gameover_mushroomavenger;
    char* gameover_impossible;
    char* console_keybind[gene_keybind_max];
    char* console_hint[gene_keybind_max];
    char* console_changekeybindF1;
    char* console_resetkeybindF1;
 //   char* console_partname[part_type_max];
    char* console_buildyourship;
    char* console_doublevalid;
    char* console_doubleescape;
    char* console_invalidmsg[invalid_reason_max];
    char* console_weight;
    char* console_power;
    char* console_agility;
    char* console_speed;
    char* console_shield;
    char* console_ram;
};

struct mech_gear;

/* Global */

int lineskip;
struct optimizations opti;
struct local local;
float global_volume; // From 0.0 to 1.0
int /*bool*/ mute;

/* Functions */

SDL_Texture* loadTexture(SDL_Renderer* renderer, char* filename);
SDL_Texture* loadTextureSize(SDL_Renderer* renderer, char* filename, int* w, int* h);
SDL_Texture* loadTextureText(SDL_Renderer* renderer, char* text, int* w, int * h, SDL_Color const * const color);
void sdl_error(char* msg);
void error(char* msg);
char* path(char* subdata_rep, char* filename);
char* path_localized(char* subdata_rep, char* filename);
int game(SDL_Renderer* renderer, int run_id, SDL_Keycode keybind[game_keybind_max]);
void menu(SDL_Renderer* renderer, SDL_Window* screen);
void gameover(SDL_Renderer* renderer, struct score score, int win);
void cinematic(SDL_Renderer* renderer, int chapter);
void briefing(SDL_Renderer* renderer, int chapter);
void generate_stars(struct star* stars, int nb_stars);
void draw_stars(SDL_Renderer* renderer, struct star* stars, int nb_stars);
SDL_Surface* build_archip(SDL_Renderer* renderer, struct gear* gear, int chapter);
void add_weapon(struct gear* gear, int x, int y, int cd, int delay, float xspeed, float yspeed, enum missile_type missile);
void add_thruster(struct gear* gear, int x, int y, int w, int h, SDL_Color const * const color_trail);
void add_special_actif(struct gear* gear, int x, int y, int cd, int delay, int duration, enum special_actif_type type);
void add_special_passif(struct gear* gear, int x, int y, enum special_passif_type type);
// void load_level(SDL_Renderer* renderer, struct run* run, int run_id);
void load_level(SDL_Renderer* renderer, struct run* run, int run_id, struct ship_model ship_models[], int nb_ships);
void load_mechip_models(SDL_Renderer* renderer, struct ship_model ship_model[], int* nb_ships);
void unload_mechip_models(int nb_ships, struct ship_model ship_model[]);
void load_briefing(char* txt[], int chapter);
void unload_briefing(char* txt[]);
int load_cinematic_text(int chapter, char* txt[], int nb_txt);
void unload_cinematic_text(char* txt[], int nb_txt);
void load_local();
void unload_local();
void begin_sound();
void stop_sound();
void add_sound(int wave_id, float volume);

#endif // MAIN_H
