
/* Copyright  2014 Pierre Vidal <p.terkal@gmail.com>
 * This program is free software. It comes without no warranty.
 * You can redistribute it and/or modify it under the terms of the
 * Do What The Fuck You Want To Public License, Version 2, as published by Sam Hocevar.
 * See the COPYING file or http://www.wtfpl.net/ for more details. */

#include "main.h"

#include <stdio.h>
#include <string.h> // only language_menu() use strcmp().

#define BANNER_Y_SHIFT 75
#define BANNER_POST_SHIFT 30
#define FONT_MULTIPL 1.5

struct menu_entry {
    SDL_Texture* texture_normal;
    SDL_Texture* texture_selected;
    int w;
    int h;
    int s_w; // Selected width.
    int s_h; // Selected height.
};

enum menu_entry_id {
    menu_entry_story,
    menu_entry_arcade,
    menu_entry_options,
    menu_entry_exit,
    menu_entry_max
};

int fullscreen;

void switch_fullscreen(SDL_Window* screen) {
    static SDL_Window* saved_screen;

    if(screen) { // Init.
        saved_screen = screen;
        fullscreen = INIT_FULLSCREEN;
        if(fullscreen) {
            SDL_SetWindowFullscreen(screen, SDL_WINDOW_FULLSCREEN);
        }
    } else { // Switch.
        if(fullscreen) {
            SDL_SetWindowFullscreen(saved_screen, 0);
            fullscreen = false;
        } else {
            SDL_SetWindowFullscreen(saved_screen, SDL_WINDOW_FULLSCREEN);
            fullscreen = true;
        }
    }
}

void story_menu(SDL_Renderer* renderer, SDL_Keycode keybind[game_keybind_max]) {
    const int nb_chapter = 2+1; // +1 is for "exit" menu entry.
    static int chapter = 1;
    int selector = chapter;
    int i;
    SDL_Event event;
    int stop = false;
    int nb_stars = rand()%30 + 70;
    struct star stars[nb_stars];
    char* title[nb_chapter];
    SDL_Texture* banner;
    SDL_Rect banner_dst;
    SDL_Texture* title_deselected[nb_chapter];
    SDL_Texture* title_selected[nb_chapter];
    SDL_Rect title_dst[nb_chapter];
    SDL_Texture* unknown_chapter_deselected;
    SDL_Texture* unknown_chapter_selected;
    SDL_Rect unknown_chapter_dst;

    if(chapter == 1) {
        briefing(renderer, 1);
        if(game(renderer, 1, keybind) == 1) {
            cinematic(renderer, 1);
            if(chapter < nb_chapter)
                chapter++;
                selector++;
        } else {
            return;
        }
    }

    // Loadings.

    title[0] = "Return to main menu";
    title[1] = "Last Sun's Light"; // Logical Shift Left
    title[2] = "Rings of Saturn's Salvation"; // Rich Site Summary
/*
    title[X] = "Rush Through the Portal"; // Real Time Protocol
    title[X] = "SuperNova's Mushroom Party";// Simple Network Managment Protocol
    title[X] = "Run for Combat"; // Request For Comments
    title[X] = "Fight to Pass"; // File Transfert Protocol
*/

    banner = loadTextureSize(renderer, path_localized("text", "banner_chapter.bmp"), &(banner_dst.w), &(banner_dst.h));
    banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
    banner_dst.y = BANNER_Y_SHIFT;
    unknown_chapter_selected = loadTextureText(renderer, "???", &(unknown_chapter_dst.w), &(unknown_chapter_dst.h), color_selected);
    unknown_chapter_deselected = loadTextureText(renderer, "???", &(unknown_chapter_dst.w), &(unknown_chapter_dst.h), color_deselected);
    unknown_chapter_dst.x = SCREEN_W/2 - unknown_chapter_dst.w/2;
    if(banner == NULL || unknown_chapter_selected == NULL || unknown_chapter_deselected == NULL)
        return;
    for(int i=0; i<nb_chapter; i++) {
        title_deselected[i] = loadTextureText(renderer, title[i], &(title_dst[i].w), &(title_dst[i].h), color_deselected);
        title_selected[i] = loadTextureText(renderer, title[i], &(title_dst[i].w), &(title_dst[i].h), color_selected);
        title_dst[i].x = SCREEN_W/2 - title_dst[i].w/2;
        title_dst[i].y = banner_dst.y + banner_dst.h + BANNER_POST_SHIFT + lineskip * (i+1);
        if(title_deselected[i] == NULL || title_selected[i] == NULL)
            return;
    }

    generate_stars(stars, nb_stars);

    // LOOP

    while(!stop) {
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                switch(event.key.keysym.sym) {
                    case SDLK_ESCAPE:
                        stop = true;
                        break;
                    case SDLK_UP:
                        if(selector > 0)
                            selector--;
                        break;
                    case SDLK_DOWN:
                        if(selector < chapter && selector < nb_chapter-1)
                            selector++;
                        break;
                    case SDLK_SPACE:
                    case SDLK_RETURN:
                    case SDLK_KP_ENTER:
                        if(selector == 0) {
                            stop = true;
                        } else {
                            briefing(renderer, selector);
                            if(game(renderer, selector, keybind) == 1) {
                                cinematic(renderer, selector);
                                if(chapter < nb_chapter)
                                    chapter++;
                            }
                        }
                        break;
                }
            }
        }

        // Draw the menu.

        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);
        draw_stars(renderer, stars, nb_stars);
        SDL_RenderCopy(renderer, banner, NULL, &banner_dst);
        for(i=0; i<chapter; i++) {
            SDL_RenderCopy(renderer, (i==selector ? title_selected[i] : title_deselected[i]), NULL, &(title_dst[i]));
        }
        if(chapter < nb_chapter) {
            unknown_chapter_dst.y = title_dst[i].y;
            SDL_RenderCopy(renderer, (i==selector ? unknown_chapter_selected : unknown_chapter_deselected), NULL, &unknown_chapter_dst);
        }
        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    SDL_DestroyTexture(banner);
    SDL_DestroyTexture(unknown_chapter_selected);
    SDL_DestroyTexture(unknown_chapter_deselected);
    for(int i=0; i<nb_chapter; i++) {
        SDL_DestroyTexture(title_deselected[i]);
        SDL_DestroyTexture(title_selected[i]);
    }
}

void keybind_menu(SDL_Renderer* renderer, SDL_Keycode kb[game_keybind_max]) {
    int stop = false;
    int waitkey = false;
    SDL_Event event;
    enum game_keybind selector = 0;
    SDL_Texture* texture;
    SDL_Rect dst;
    SDL_Texture* banner;
    SDL_Rect banner_dst;
    char txt[BUFSIZ];
    char * bindname;
    SDL_Texture* hint[MAX_KEYBIND_HINT];
    SDL_Rect hint_dst[MAX_KEYBIND_HINT];

    // Loading.
    for(int i=0; i<MAX_KEYBIND_HINT; i++) {
        hint[i] = loadTextureText(renderer, local.keybindmenu_hints[i], &(hint_dst[i].w), &(hint_dst[i].h), color_hint);
    }
    for(int i=0; i<MAX_KEYBIND_HINT; i++) {
        hint_dst[i].x = SCREEN_W - hint_dst[i].w - 50;
        hint_dst[i].y = SCREEN_H - (lineskip*FONT_MULTIPL * (MAX_KEYBIND_HINT-i)) - 50;
    }
    banner = loadTextureSize(renderer, path_localized("text" , "keybind.bmp"), &(banner_dst.w), &(banner_dst).h);
    if(banner == NULL)
        return;
    banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
    banner_dst.y = BANNER_Y_SHIFT;

    while(!stop) {
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                if(waitkey) {
                    if(event.key.keysym.sym == SDLK_ESCAPE) {
                        waitkey = false;
                    } else {
                        kb[selector] = event.key.keysym.sym;
                        waitkey = false;
                    }
                } else {
                    switch(event.key.keysym.sym) {
                        case SDLK_ESCAPE:
                            stop = true;
                            break;
                        case SDLK_UP:
                            if(selector > 0)
                                selector--;
                            break;
                        case SDLK_DOWN:
                            if(selector < game_keybind_max-1)
                                selector++;
                            break;
                        case SDLK_SPACE:
                        case SDLK_RETURN:
                        case SDLK_KP_ENTER:
                            waitkey = true;
                            break;
                    }
                }
            }
        }

        // Draw keybind state.

        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);

        for(enum game_keybind i=0; i<game_keybind_max; i++) {
            bindname = local.keybindmenu_entry[i];
            sprintf(txt, "%s : %s", bindname, SDL_GetKeyName(kb[i]));
            texture = loadTextureText(renderer, txt, &(dst.w), &(dst.h), selector == i ? (waitkey ? color_waiting : color_selected) : color_deselected);
            dst.w *= FONT_MULTIPL;
            dst.h *= FONT_MULTIPL;
            dst.x = 50;
            dst.y = banner_dst.y + banner_dst.h + BANNER_POST_SHIFT + i*lineskip*FONT_MULTIPL;
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
        }

        SDL_RenderCopy(renderer, banner, NULL, &banner_dst);
        for(int i=0; i<MAX_KEYBIND_HINT; i++) {
            SDL_RenderCopy(renderer, hint[i], NULL, &(hint_dst[i]));
        }
        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    // Clean up.
    SDL_DestroyTexture(banner);
    for(int i=0; i<MAX_KEYBIND_HINT; i++) {
        SDL_DestroyTexture(hint[i]);
    }
}

void display_menu(SDL_Renderer* renderer) {
    int stop = false;
    SDL_Texture* banner;
    SDL_Rect banner_dst;
    SDL_Texture* texture;
    SDL_Rect dst;
    SDL_Event event;
    char* txt;
    int boolean;
    enum display_entry_id selector = 0;

    // Load textures.
    banner = loadTextureSize(renderer, path_localized("text", "display.bmp"), &(banner_dst.w), &(banner_dst.h));
    banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
    banner_dst.y = BANNER_Y_SHIFT;

    // Check texture loading.
    if(banner == NULL)
        return;

    while(!stop) {

        // Process input.
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                switch(event.key.keysym.sym) {
                    case SDLK_ESCAPE:
                        stop = true;
                        break;
                    case SDLK_UP:
                        if(selector > 0)
                            selector--;
                        break;
                    case SDLK_DOWN:
                        if(selector < display_entry_max-1)
                            selector++;
                        break;
                    case SDLK_RETURN:
                    case SDLK_KP_ENTER:
                    case SDLK_SPACE:
                        switch(selector) {
                            case display_entry_return:
                                stop = true;
                                break;
                            case display_entry_fullscreen:
                                switch_fullscreen(NULL);
                                break;
                            case display_entry_trail:
                                if(opti.trails)
                                    opti.trails = false;
                                else
                                    opti.trails = true;
                                break;
                            case display_entry_explosion:
                                if(opti.explosion)
                                    opti.explosion = false;
                                else
                                    opti.explosion = true;
                                break;
                            case display_entry_particles:
                                if(opti.particles)
                                    opti.particles = false;
                                else
                                    opti.particles = true;
                                break;
                            case display_entry_max:
                                break;
                        }
                }
            }
        }

        // Draw screen.
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);
        SDL_RenderCopy(renderer, banner, NULL, &banner_dst);

        for(enum display_entry_id i=0; i<display_entry_max; i++) {
            // Draw entry name.
            txt = local.optimenu_entry[i];
            switch(i) {
                case display_entry_return: boolean = -1; break;
                case display_entry_fullscreen: boolean = fullscreen; break;
                case display_entry_trail: boolean = opti.trails; break;
                case display_entry_explosion: boolean = opti.explosion; break;
                case display_entry_particles: boolean = opti.particles; break;
                case display_entry_max: break;
            }
            texture = loadTextureText(renderer, txt, &(dst.w), &(dst.h), i == selector ? color_selected : color_deselected);
            dst.w *= FONT_MULTIPL;
            dst.h *= FONT_MULTIPL;
            dst.x = SCREEN_W/2 - (boolean == -1 ? dst.w/2 : dst.w);
            dst.y = banner_dst.y + BANNER_POST_SHIFT + banner_dst.h + lineskip*FONT_MULTIPL*(i+1);
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
            // Draw entry state.
            if(boolean != -1) {
                texture = loadTextureText(renderer, boolean ? local.optimenu_enable : local.optimenu_disable, &(dst.w), &(dst.h), boolean ? color_true : color_false);
                dst.w *= FONT_MULTIPL;
                dst.h *= FONT_MULTIPL;
                dst.x = SCREEN_W/2;
                dst.y = banner_dst.y + BANNER_POST_SHIFT + banner_dst.h + lineskip*FONT_MULTIPL*(i+1);
                SDL_RenderCopy(renderer, texture, NULL, &dst);
                SDL_DestroyTexture(texture);
            }
        }

        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    // Clean up.
    SDL_DestroyTexture(banner);
}

void sound_menu(SDL_Renderer* renderer) {
    int stop = false;
    SDL_Texture* banner;
    SDL_Rect banner_dst;
    SDL_Texture* texture;
    SDL_Rect dst;
    SDL_Event event;
    enum sound_entry_id selector = 0;

    // Load texture.
    banner = loadTextureSize(renderer, path_localized("text", "sound.bmp"), &(banner_dst.w), &(banner_dst.h));
    banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
    banner_dst.y = BANNER_Y_SHIFT;

    // Check texture loading.
    if(banner == NULL)
        return;

    while(!stop) {
        // Process Input.
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                switch(event.key.keysym.sym) {
                    case SDLK_ESCAPE:
                        stop = true;
                        break;
                    case SDLK_UP:
                        if(selector > 0)
                            selector--;
                        break;
                    case SDLK_DOWN:
                        if(selector < sound_entry_max)
                            selector++;
                        break;
                    case SDLK_RETURN:
                        switch(selector) {
                            case sound_entry_return:
                                stop = 1;
                                break;
                            case sound_entry_mute:
                                if(mute) {
                                    mute = false;
                                } else {
                                    mute = true;
                                }
                                break;
                            case sound_entry_volume:
                                if(global_volume >= 1.0) {
                                    global_volume = 0.1;
                                } else {
                                    global_volume += 0.1;
                                }
                                break;
                            case sound_entry_max:
                                break;
                        }
                }
            }
        }

        // Draw screen.
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);
        SDL_RenderCopy(renderer, banner, NULL, &banner_dst);

        for(enum sound_entry_id i=0; i<sound_entry_max; i++) {
            texture = loadTextureText(renderer, local.soundmenu_entry[i], &(dst.w), &(dst.h),
                        i == selector ? color_selected : color_deselected);
            dst.w *= FONT_MULTIPL;
            dst.h *= FONT_MULTIPL;
            dst.y = banner_dst.y + BANNER_POST_SHIFT + banner_dst.h + lineskip*FONT_MULTIPL*(i+1);
            if(i == sound_entry_mute) {
                dst.x = SCREEN_W/2 - dst.w;
                SDL_RenderCopy(renderer, texture, NULL, &dst);
                SDL_DestroyTexture(texture);
                texture = loadTextureText(renderer, mute ? local.optimenu_disable : local.optimenu_enable, &(dst.w), &(dst.h),
                                          mute ? color_false : color_true);
                dst.w *= FONT_MULTIPL;
                dst.h *= FONT_MULTIPL;
                // dst.y = dst.y;
                dst.x = SCREEN_W/2;
            } else if(i == sound_entry_volume) {
                char buf[12];
                for(int i=0; i<11; i++) {
                    if(i <= global_volume*10) {
                        buf[i] = '#';
                    } else {
                        buf[i] = '_';
                    }
                }
                buf[11] = '\0';
                dst.x = SCREEN_W/2 - dst.w;
                SDL_RenderCopy(renderer, texture, NULL, &dst);
                SDL_DestroyTexture(texture);
                texture = loadTextureText(renderer, buf, &(dst.w), &(dst.h), color_hint);
                dst.w *= FONT_MULTIPL;
                dst.h *= FONT_MULTIPL;
                dst.x = SCREEN_W/2;
            } else {
                dst.x = SCREEN_W/2 - dst.w/2;
            }
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
        }

        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    // Clean up.
    SDL_DestroyTexture(banner);
}

char* langcode2countryname(char const * code) {
    if(!strcmp(code, "EN")) {
        return("English");
    } else if(!strcmp(code, "FR")) {
        return("Franais");
    } else {
        return("???");
    }
}

void language_menu(SDL_Renderer* renderer) {
    int stop = false;
    SDL_Texture* banner;
    SDL_Rect banner_dst;
    SDL_Texture* texture;
    SDL_Rect dst;
    SDL_Event event;
    int selector = 0;

    // Load texture.
    banner = loadTextureSize(renderer, path_localized("text", "language.bmp"), &(banner_dst.w), &(banner_dst.h));
    banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
    banner_dst.y = BANNER_Y_SHIFT;

    // Check texture loading.
    if(banner == NULL)
        return;

    while(!stop) {
        // Process Input.
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                switch(event.key.keysym.sym) {
                    case SDLK_ESCAPE:
                        stop = true;
                        break;
                    case SDLK_UP:
                        if(selector > 0)
                            selector--;
                        break;
                    case SDLK_DOWN:
                        if(lang_code[selector+1-1][0] != '\0')
                            selector++;
                        break;
                    case SDLK_RETURN:
                        if(selector == 0) {
                            stop = 1;
                        } else {
                            unload_local();
                            load_local(lang_code[selector-1]);
                            SDL_DestroyTexture(banner);
                            banner = loadTextureSize(renderer, path_localized("text", "language.bmp"), &(banner_dst.w), &(banner_dst.h));
                            banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
                            banner_dst.y = BANNER_Y_SHIFT;
                            break;
                        }
                }
            }
        }

        // Draw screen.
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);
        SDL_RenderCopy(renderer, banner, NULL, &banner_dst);

        texture = loadTextureText(renderer, local.optionmenu_entry[0] , &(dst.w), &(dst.h),
                    selector == 0 ? color_selected : color_deselected);
        dst.w *= FONT_MULTIPL;
        dst.h *= FONT_MULTIPL;
        dst.x = SCREEN_W/2 - dst.w/2;
        dst.y = banner_dst.y + BANNER_POST_SHIFT + banner_dst.h;
        SDL_RenderCopy(renderer, texture, NULL, &dst);
        SDL_DestroyTexture(texture);
        for(int i=0; lang_code[i][0] != '\0'; i++) {
            texture = loadTextureText(renderer, langcode2countryname(lang_code[i]), &(dst.w), &(dst.h),
                    selector == i+1 ? color_selected : (!strcmp(lang_code[i], local.code)) ? color_true : color_false);
            dst.w *= FONT_MULTIPL;
            dst.h *= FONT_MULTIPL;
            dst.x = SCREEN_W/2 - dst.w/2;
            dst.y = banner_dst.y + BANNER_POST_SHIFT + banner_dst.h + lineskip*FONT_MULTIPL*(i+1);
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
        }

        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    // Clean up.
    SDL_DestroyTexture(banner);
}

void options_menu(SDL_Renderer* renderer, SDL_Keycode keybind[game_keybind_max]) {
    int stop = false;
    SDL_Texture* banner;
    SDL_Rect banner_dst;
    SDL_Texture* texture;
    SDL_Rect dst;
    SDL_Event event;
    char* txt;
    enum option_entry_id selector = 0;

    // Load textures.
    banner = loadTextureSize(renderer, path_localized("text", "options.bmp"), &(banner_dst.w), &(banner_dst.h));
    banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
    banner_dst.y = BANNER_Y_SHIFT;

    // Check texture loading.
    if(banner == NULL)
        return;

    while(!stop) {

        // Process input.
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT) {
                stop = true;
            } else if(event.type == SDL_KEYDOWN) {
                switch(event.key.keysym.sym) {
                    case SDLK_ESCAPE:
                        stop = true;
                        break;
                    case SDLK_UP:
                        if(selector > 0)
                            selector--;
                        break;
                    case SDLK_DOWN:
                        if(selector < option_entry_max-1)
                            selector++;
                        break;
                    case SDLK_RETURN:
                    case SDLK_KP_ENTER:
                    case SDLK_SPACE:
                        switch(selector) {
                            case option_entry_return:
                                stop = true;
                                break;
                            case option_entry_controls:
                                keybind_menu(renderer, keybind);
                                break;
                            case option_entry_display:
                                display_menu(renderer);
                                break;
                            case option_entry_sound:
                                sound_menu(renderer);
                                break;
                            case option_entry_language:
                                language_menu(renderer);
                                break;
                            case option_entry_max:
                                break;
                        }
                        break;
                }
            }
        }

        // Draw screen.
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);
        SDL_RenderCopy(renderer, banner, NULL, &banner_dst);

        for(enum option_entry_id i=0; i<option_entry_max; i++) {
            txt = local.optionmenu_entry[i];
            texture = loadTextureText(renderer, txt, &(dst.w), &(dst.h), i == selector ? color_selected : color_deselected);
            dst.w *= FONT_MULTIPL;
            dst.h *= FONT_MULTIPL;
            dst.x = SCREEN_W/2 - dst.w/2;
            dst.y = banner_dst.y + banner_dst.h + BANNER_POST_SHIFT + lineskip*FONT_MULTIPL*(i+1);
            SDL_RenderCopy(renderer, texture, NULL, &dst);
            SDL_DestroyTexture(texture);
        }

        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    // Clean up.
    SDL_DestroyTexture(banner);
}

void menu(SDL_Renderer* renderer, SDL_Window* screen) {
    SDL_Texture* title;
    SDL_Texture* hint;
    SDL_Texture* version;
    SDL_Texture* pumpkin;
    SDL_Event event;
    int stop = false; // Main loop's stop flag.
    char* filename;
    enum menu_entry_id selector = 0;
    SDL_Rect title_dst, hint_dst, version_dst, pumpkin_dst;
    struct menu_entry menu[menu_entry_max];
    int nb_stars = rand()%30 + 70;
    struct star stars[nb_stars];
    SDL_Keycode game_keybind[game_keybind_max];

    generate_stars(stars, nb_stars);

    // Load textures.

    title = loadTextureSize(renderer, path_localized("text", "banner_title.bmp"), &(title_dst.w), &(title_dst.h));
    hint = loadTextureSize(renderer, path_localized("text", "usearrowkeys.bmp"), &(hint_dst.w), &(hint_dst.h));
    pumpkin = loadTextureSize(renderer, path("", "pumpkin.bmp"), &(pumpkin_dst.w), &(pumpkin_dst.h));
    for(enum menu_entry_id i=0; i<menu_entry_max; i++) {
        switch(i) {
            case menu_entry_arcade: filename = path_localized("text", "arcade_mush.bmp");; break;
            case menu_entry_story: filename = path_localized("text", "story_mush.bmp"); break;
            case menu_entry_exit: filename = path_localized("text", "exit_mush.bmp"); break;
            case menu_entry_options: filename = path_localized("text", "options_mush.bmp"); break;
            case menu_entry_max: break;
        }
        menu[i].texture_normal = loadTextureSize(renderer, filename, &(menu[i].w), &(menu[i].h));
        switch(i) {
            case menu_entry_arcade: filename = path_localized("text", "arcade.bmp"); break;
            case menu_entry_story: filename = path_localized("text", "story.bmp"); break;
            case menu_entry_exit: filename = path_localized("text", "exit.bmp"); break;
            case menu_entry_options: filename = path_localized("text", "options.bmp"); break;
            case menu_entry_max: break;
        }
        menu[i].texture_selected = loadTextureSize(renderer, filename, &(menu[i].s_w), &(menu[i].s_h));
    }

    // Check texture loading.

    if(title == NULL || hint == NULL || pumpkin == NULL) {
        return;
    }
    for(int i=0; i<menu_entry_max; i++) {
        if(menu[i].texture_normal == NULL || menu[i].texture_selected == NULL) {
            return;
        }
    }

    title_dst.x = SCREEN_W/2 - title_dst.w/2;
    title_dst.y = 20;
    hint_dst.x = SCREEN_W - 20 - hint_dst.w;
    hint_dst.y = SCREEN_H - 20 - hint_dst.h;

    // Build version texture.
    version = loadTextureText(renderer, VERSION, &(version_dst.w), &(version_dst.h), color_hint);
    version_dst.x = 20;
    version_dst.y = SCREEN_H - version_dst.h - 20;
    pumpkin_dst.x = 20;
    pumpkin_dst.y = version_dst.y - 20 - pumpkin_dst.h;

    // Build default game keybinding.
    game_keybind[game_keybind_moveup] = SDLK_UP;
    game_keybind[game_keybind_movedown] = SDLK_DOWN;
    game_keybind[game_keybind_moveleft] = SDLK_LEFT;
    game_keybind[game_keybind_moveright] = SDLK_RIGHT;
    game_keybind[game_keybind_fire] = SDLK_SPACE;
    game_keybind[game_keybind_altfire] = SDLK_LCTRL;

    // Set fullscreen window.
    switch_fullscreen(screen);

    while(!stop) {
        SDL_Rect dst;

        // Draw menu.

        // Clear screen.
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);

        // Draw stars.
        draw_stars(renderer, stars, nb_stars);

        SDL_RenderCopy(renderer, title, NULL, &title_dst);
        SDL_RenderCopy(renderer, hint, NULL, &hint_dst);
        SDL_RenderCopy(renderer, version, NULL, &version_dst);
        SDL_RenderCopy(renderer, pumpkin, NULL, &pumpkin_dst);

        // Draw entries.
        int i;
        for(i=0, dst.y=title_dst.y+title_dst.h+100; i<menu_entry_max; i++, dst.y+=70) {
            if(selector == i) {
                dst.w = menu[i].s_w;
                dst.h = menu[i].s_h;
                dst.x = SCREEN_W/2 - menu[i].s_w/2;
                SDL_RenderCopy(renderer, menu[i].texture_selected, NULL, &dst);
            } else {
                dst.w = menu[i].w;
                dst.h = menu[i].h;
                dst.x = SCREEN_W/2 - menu[i].w/2;
                SDL_RenderCopy(renderer, menu[i].texture_normal, NULL, &dst);
            }
        }

        // Draw version.

        SDL_RenderPresent(renderer);

        // Get input.

        if(SDL_WaitEvent(&event) != 1) {
            sdl_error("Error in menu waiting for keyboard event");
            return;
        }
        if(event.type == SDL_QUIT) {
            stop = 1;
        } else if(event.type == SDL_KEYDOWN) {
            switch(event.key.keysym.sym) {
                case SDLK_ESCAPE:
                    stop = 1;
                    break;
                case SDLK_DOWN:
                    if(selector < menu_entry_max-1)
                        selector++;
                    break;
                case SDLK_UP:
                    if(selector > 0)
                        selector--;
                    break;
                case SDLK_RETURN:
                case SDLK_SPACE:
                case SDLK_KP_ENTER:
                    switch(selector) {
                        case menu_entry_exit:
                            stop = 1;
                            break;
                        case menu_entry_arcade:
                            game(renderer, ARCADE_LEVEL, game_keybind);
                            break;
                        case menu_entry_story:
                            story_menu(renderer, game_keybind);
                            break;
                        case menu_entry_options:
                            options_menu(renderer, game_keybind);
                            break;
                        case menu_entry_max:
                            break;
                    }
                    break;
                case SDLK_F11:
                    switch_fullscreen(NULL);
                    break;
            }
        }

        SDL_Delay(10);
    }

    // Clean up.
    SDL_DestroyTexture(title);
    SDL_DestroyTexture(hint);
    for(int i=0; i<menu_entry_max; i++) {
        SDL_DestroyTexture(menu[i].texture_normal);
        SDL_DestroyTexture(menu[i].texture_selected);
    }
}

void gameover(SDL_Renderer* renderer, struct score score, int win) {
    SDL_Texture* banner;
    SDL_Texture* hint;
    SDL_Texture* score_texture;
    SDL_Rect banner_dst, hint_dst, score_dst;
    char score_txt[32];
    int nb_stars = rand()%30 + 70;
    struct star stars[nb_stars];

    // score.time /= 100; // Scale time in seconds.
    score.time /= 10; // Scale time in 10th of a second.

    // Load textures.
    if(win) {
        banner = loadTextureSize(renderer, path_localized("text", "banner_escaped.bmp"), &(banner_dst.w), &(banner_dst.h));
        generate_stars(stars, nb_stars);
    } else {
        banner = loadTextureSize(renderer, path_localized("text", "banner_gameover.bmp"), &(banner_dst.w), &(banner_dst.h));
        nb_stars = 0;
    }
    hint = loadTextureText(renderer, local.gameover_pressenter, &(hint_dst.w), &(hint_dst.h), color_hint);

    // Check texture loading.
    if(banner == NULL || hint == NULL) {
        sdl_error("Unable to load text");
        return;
    }

    // Clear screen.
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
    SDL_RenderClear(renderer);

    // Draw.

    draw_stars(renderer, stars, nb_stars);

    banner_dst.x = SCREEN_W/2 - banner_dst.w/2;
    banner_dst.y = BANNER_Y_SHIFT;
    SDL_RenderCopy(renderer, banner, NULL, &banner_dst);

    // Draw scores.
    for(enum score_entry i=0; i<=score_entry_max; i++) {
        int* iptr = NULL;
        char* cptr = NULL;

        if(i == score_entry_max) {
            cptr = local.gameover_accuracy;
        } else {
            cptr = local.gameover_scores[i];
        }
        switch(i) {
            case score_destroy: iptr = &(score.nb_destroy); break;
            case score_dodge: iptr = &(score.nb_dodge); break;
            case score_time: iptr = &(score.time); break;
            case score_shot: iptr = &(score.nb_shot); break;
            case score_hit: iptr = &(score.nb_hit); break;
            case score_entry_max: break;
        }
        if(i == score_entry_max) {
            if(score.nb_shot <= 0) {
                sprintf(score_txt, " : -");
            } else {
                sprintf(score_txt, " : %d%%", score.nb_hit*100/score.nb_shot);
            }
        } else {
            sprintf(score_txt, " : %d", *iptr);
        }
        score_dst.y = banner_dst.y + BANNER_POST_SHIFT + lineskip*FONT_MULTIPL*(i+1);
        score_texture = loadTextureText(renderer, cptr, &(score_dst.w), &(score_dst.h), color_hint);
        score_dst.w *= FONT_MULTIPL;
        score_dst.h *= FONT_MULTIPL;
        score_dst.x = SCREEN_W*3/5 - score_dst.w;
        SDL_RenderCopy(renderer, score_texture, NULL, &score_dst);
        SDL_DestroyTexture(score_texture);
        score_texture = loadTextureText(renderer, score_txt, &(score_dst.w), &(score_dst.h), color_special);
        score_dst.w *= FONT_MULTIPL;
        score_dst.h *= FONT_MULTIPL;
        score_dst.x = SCREEN_W*3/5;
        SDL_RenderCopy(renderer, score_texture, NULL, &score_dst);
        SDL_DestroyTexture(score_texture);
    }

    // Draw achievments.
    if(win) {
        score_dst.y = banner_dst.y+ BANNER_POST_SHIFT + lineskip*FONT_MULTIPL*(score_entry_max+1);
        if(score.nb_shot > 0 && score.nb_shot*3 <= score.nb_hit*4) {
            score_dst.y += lineskip*FONT_MULTIPL;
            score_texture = loadTextureText(renderer, score.nb_shot <= score.nb_hit ? local.gameover_ult_geneticefficiency : local.gameover_geneticefficiency, &(score_dst.w), &(score_dst.h), &color_violet);
            score_dst.w *= FONT_MULTIPL;
            score_dst.h *= FONT_MULTIPL;
            score_dst.x = SCREEN_W/2 - score_dst.w/2;
            SDL_RenderCopy(renderer, score_texture, NULL, &score_dst);
            SDL_DestroyTexture(score_texture);
        }
        if(score.nb_destroy > 0 && score.nb_destroy >= score.nb_dodge*2) {
            score_dst.y += lineskip*FONT_MULTIPL;
            score_texture = loadTextureText(renderer, score.nb_dodge <= 0 ? local.gameover_ult_mushroomavenger : local.gameover_mushroomavenger, &(score_dst.w), &(score_dst.h), &color_violet);
            score_dst.w *= FONT_MULTIPL;
            score_dst.h *= FONT_MULTIPL;
            score_dst.x = SCREEN_W/2 - score_dst.w/2;
            SDL_RenderCopy(renderer, score_texture, NULL, &score_dst);
            SDL_DestroyTexture(score_texture);
        }
        if(score.nb_shot > 0 && score.nb_destroy > 0 && score.nb_dodge <= 0 && score.nb_shot <= score.nb_hit) {
            score_dst.y += lineskip*FONT_MULTIPL;
            score_texture = loadTextureText(renderer, local.gameover_impossible, &(score_dst.w), &(score_dst.h), color_evil);
            score_dst.w *= FONT_MULTIPL;
            score_dst.h *= FONT_MULTIPL;
            score_dst.x = SCREEN_W/2 - score_dst.w/2;
            SDL_RenderCopy(renderer, score_texture, NULL, &score_dst);
            SDL_DestroyTexture(score_texture);
        }
    }

    // Draw hint.
    hint_dst.w *= 2;
    hint_dst.h *= 2;
    hint_dst.x = SCREEN_W - hint_dst.w - 10;
    hint_dst.y = SCREEN_H - hint_dst.h - 10;
    SDL_RenderCopy(renderer, hint, NULL, &hint_dst);

    int stop = false;
    while(!stop) {
        SDL_Event event;
        while(SDL_PollEvent(&event)) {
            if(event.type == SDL_QUIT ||
                    (event.type == SDL_KEYDOWN &&
                        (event.key.keysym.sym == SDLK_ESCAPE ||
                         event.key.keysym.sym == SDLK_KP_ENTER ||
                         event.key.keysym.sym == SDLK_RETURN)
                     )
            ) {
                stop = true;
            }
        }
        SDL_RenderPresent(renderer);
        SDL_Delay(10);
    }

    // Clean up.
    SDL_DestroyTexture(banner);
    SDL_DestroyTexture(hint);
}
