
/* Copyright  2014 Pierre Vidal <p.terkal@gmail.com>
 * This program is free software. It comes without no warranty.
 * You can redistribute it and/or modify it under the terms of the
 * Do What The Fuck You Want To Public License, Version 2, as published by Sam Hocevar.
 * See the COPYING file or http://www.wtfpl.net/ for more details. */

#include "main.h"

#include <string.h> // memset().
#include <stdio.h> // sprintf().

#define MAX_SOUND     32
#define MAX_AUDIOWAVE 8

struct audiowave {
    unsigned char * buffer;
    unsigned int length;
};

SDL_AudioDeviceID dev;
SDL_AudioSpec audio_spec;
unsigned char * sound_ptr;
unsigned int sound_remains;
struct audiowave wav_lst[MAX_WAVE];
float sound_volume;

/* http://stackoverflow.com/questions/12089662/mixing-16-bit-linear-pcm-streams-and-avoiding-clipping-overflow
short int mix_sample(short int sample1, short int sample2) {
    sample1/2 + sample2/2
    2*(sample1 + sample2) - 2*(sample1*sample2) - 65535
    (sample1 + sample2) - sample1*sample2
    (sample1 + sample2) - sample1*sample2 - 65535
    (sample1 + sample2) - ((sample1*sample2) >> 0x10) // same as divide by 65535
}
*/

void sound_callback(void * userdata, unsigned char * stream, int total_length) {
    // SDL_memset(stream, 0, total_length);
    memset(stream, 0, total_length);

    if(sound_remains > 0) {
        int length = total_length > sound_remains ? sound_remains : total_length;
        float volume = sound_volume * global_volume;
        for(int i=0; i<length; i++) {
            stream[i] = sound_ptr[i] * volume;
        }
        // SDL_MixAudioFormat(stream, sound_ptr, length, audio_spec.format, volume);
        sound_ptr += length;
        sound_remains -= length;
    }
}

char* soundid2filename(enum sound_id id) {
    switch(id) {
        case sound_explosion: return("explosion.wav"); break;
        case sound_explosion_grave: return("explosion_grave.wav"); break;
        case sound_thruster_doppler: return("thruster_doppler.wav"); break;
        /*
        case sound_thruster_mush: return("thruster_mush.wav"); break;
        case sound_thruster_mech: return("thruster_mech.wav"); break;
        */
        case sound_id_max: break;
    }
    return("");
}

void load_audiowave() {
    for(enum sound_id i=0; i<sound_id_max; i++) {
        char* filename = soundid2filename(i);
        if(SDL_LoadWAV(path("sound", filename), &audio_spec, &(wav_lst[i].buffer), &(wav_lst[i].length)) == NULL) {
            char buf[BUFSIZ];
            sprintf(buf, "Unable to open audio file '%s'", filename);
            sdl_error(buf);
            wav_lst[i].buffer = NULL;
        }
    }
}

void unload_audiowave() {
    for(enum sound_id i=0; i<sound_id_max; i++) {
        if(wav_lst[i].buffer != NULL) {
            SDL_FreeWAV(wav_lst[i].buffer);
        }
    }
}

void begin_sound() {
    sound_ptr = NULL;
    sound_remains = 0;
    sound_volume = 0;

    SDL_zero(audio_spec);
    audio_spec.freq = 48000;
    audio_spec.format = AUDIO_U8; // 8bit POWAAA!!!
    audio_spec.channels = 1; // For now...
    audio_spec.samples = 4096;
    audio_spec.callback = sound_callback;
    audio_spec.userdata = NULL;

    if((dev = SDL_OpenAudioDevice(NULL, 0, &audio_spec, NULL, SDL_AUDIO_ALLOW_ANY_CHANGE)) <= 0) {
        sdl_error("Unable to open audio device");
    } else {
        load_audiowave();
        SDL_PauseAudioDevice(dev, 0);
    }
}

void stop_sound() {
    SDL_PauseAudioDevice(dev, 1);
    unload_audiowave();
    SDL_CloseAudioDevice(dev);
}

void add_sound(int wave_id, float volume) {
    sound_ptr = wav_lst[wave_id].buffer;
    sound_remains = wav_lst[wave_id].length;
    sound_volume = volume;
}
